@@ -27,7 +27,7 @@ from piper.verbs import non_alpha
 from piper.verbs import order_by
 from piper.verbs import outer_join
 from piper.verbs import overlaps
-from piper.verbs import pivot_wider
+from piper.verbs import pivot_table
 from piper.verbs import relocate
 from piper.verbs import rename
 from piper.verbs import rename_axis
@@ -253,41 +253,44 @@ def test_assign(t_sample_data):
     assert expected == actual
 
 
-# test_assign_with_str_formulas {{{1
-def test_assign_with_str_formulas(t_sample_data):
+# test_assign_with_dataframe_object_formulas {{{1
+def test_assign_with_dataframe_object_formulas(t_sample_data):
     """
     """
     df = t_sample_data
     df = where(df, "ids == 'A'")
     df = where(df, "values_1 > 300 & countries.isin(['Italy', 'Spain'])")
-    df = assign(df, new_field='x.countries.str[:3]+x.regions',
-                       another='3*x.values_1', lambda_str=True, info=True)
+    df = assign(df, new_field=lambda x: x.countries.str[:3] + x.regions,
+                    another=lambda x: 3*x.values_1)
 
-    expected = ['dates', 'order_dates', 'countries',
-                'regions', 'ids', 'values_1',
+    expected = ['dates', 'order_dates', 'countries', 'regions', 'ids', 'values_1',
                 'values_2', 'new_field', 'another']
     actual = df.columns.tolist()
 
     assert expected == actual
 
+# test_assign_with_tuple_function {{{1
+def test_assign_with_tuple_function(t_sample_data):
 
-# test_assign_with_str_formulas_info {{{1
-def test_assign_with_str_formulas_info(t_sample_data):
-    """
-    """
     df = t_sample_data
-    df = where(df, "ids == 'A'")
-    df = where(df, "values_1 > 300 & countries.isin(['Italy', 'Spain'])")
-    df = assign(df, new_field='x.countries.str[:3]+x.regions',
-                       another='3*x.values_1', info=True)
+    df = assign(df, reversed=('regions', lambda x: x[::-1]))
+    df = select(df, ['-dates', '-order_dates'])
 
-    expected = ['dates', 'order_dates', 'countries',
-                'regions', 'ids', 'values_1',
-                'values_2', 'new_field', 'another']
+    expected = ['countries', 'regions', 'ids', 'values_1', 'values_2', 'reversed']
     actual = df.columns.tolist()
 
     assert expected == actual
 
+
+# test_assign_with_value_error {{{1
+def test_assign_with_value_error(t_sample_data):
+
+    df = t_sample_data
+
+    with pytest.raises(ValueError):
+        actual = assign(df, reversed=lambda x: x[::-1])
+
+
 # test_clean_column_list_using_list_and_title {{{1
 def test_clean_column_list_using_list_and_title(get_column_list):
     """
@@ -811,7 +814,7 @@ def test_info(t_simple_series_01):
 
     df = t_simple_series_01.to_frame()
 
-    expected = (1, 6)
+    expected = (1, 7)
     actual = info(df).shape
 
     assert expected == actual
@@ -821,7 +824,7 @@ def test_info_with_dupes(t_simple_series_01):
 
     df = t_simple_series_01.to_frame()
 
-    expected = (1, 7)
+    expected = (1, 8)
     actual = info(df, n_dupes=True).shape
 
     assert expected == actual
@@ -831,7 +834,7 @@ def test_info_with_na_cols(t_simple_series_01):
 
     df = t_simple_series_01.to_frame()
 
-    expected = (0, 6)
+    expected = (0, 7)
     actual = info(df, fillna=True).shape
 
     assert expected == actual
@@ -963,14 +966,14 @@ def test_overlaps_unique_key_list():
     assert expected == actual.shape
 
 
-# test_pivot_wider {{{1
-def test_pivot_wider(t_sample_data):
+# test_pivot_table {{{1
+def test_pivot_table(t_sample_data):
     """
     """
 
     df = t_sample_data
 
-    pv = pivot_wider(df, index=['countries', 'regions'], values='values_1')
+    pv = pivot_table(df, index=['countries', 'regions'], values='values_1')
     pv.rename(columns={'values_1': 'totals'}, inplace=True)
 
     expected = 6507.9683290565645
@@ -978,14 +981,14 @@ def test_pivot_wider(t_sample_data):
 
     assert expected == actual
 
-# test_pivot_wider_sort_ascending_false {{{1
-def test_pivot_wider_sort_ascending_false(t_sample_data):
+# test_pivot_table_sort_ascending_false {{{1
+def test_pivot_table_sort_ascending_false(t_sample_data):
     """
     """
 
     df = t_sample_data
 
-    pv = pivot_wider(df, index=['countries'], values='values_1')
+    pv = pivot_table(df, index=['countries'], values='values_1')
     pv.sort_values(by='values_1', ascending=False, inplace=True)
 
     expected = (8, 1)
@@ -1002,7 +1005,7 @@ def test_pivot_name_error(t_sample_data):
     df = t_sample_data
 
     with pytest.raises(KeyError):
-        pv = pivot_wider(df, index=['countries_wrong_name'], values='values_1')
+        pv = pivot_table(df, index=['countries_wrong_name'], values='values_1')
 
 # test_pivot_percent_calc {{{1
 def test_pivot_percent_calc(t_sample_data):
@@ -1011,7 +1014,7 @@ def test_pivot_percent_calc(t_sample_data):
 
     df = t_sample_data
 
-    pv = pivot_wider(df, index=['countries', 'regions'], values='values_1')
+    pv = pivot_table(df, index=['countries', 'regions'], values='values_1')
     pv.rename(columns={'values_1': 'totals'}, inplace=True)
     pv.sort_values(by='totals', ascending=False, inplace=True)
     pv['%'] = pv.totals.apply(lambda x: x*100/pv.totals.sum())
@@ -1027,7 +1030,7 @@ def test_pivot_cum_percent_calc(t_sample_data):
     """
     df = t_sample_data
 
-    pv = pivot_wider(df, index=['countries', 'regions'], values='values_1')
+    pv = pivot_table(df, index=['countries', 'regions'], values='values_1')
     pv.rename(columns={'values_1': 'totals'}, inplace=True)
     pv.sort_values(by='totals', ascending=False, inplace=True)
     pv['%'] = pv.totals.apply(lambda x: x*100/pv.totals.sum())
@@ -1039,13 +1042,13 @@ def test_pivot_cum_percent_calc(t_sample_data):
     assert expected == actual
 
 
-# test_pivot_wider_multi_grouper {{{1
-def test_pivot_wider_multi_grouper(t_sample_data):
+# test_pivot_table_multi_grouper {{{1
+def test_pivot_table_multi_grouper(t_sample_data):
     """
     """
     df = t_sample_data
 
-    p2 = pivot_wider(df, index=['dates', 'order_dates',
+    p2 = pivot_table(df, index=['dates', 'order_dates',
                                 'regions', 'ids'],
                                 freq='Q',
                                 format_date=True)
@@ -1053,13 +1056,13 @@ def test_pivot_wider_multi_grouper(t_sample_data):
     assert p2.loc[('Mar 2020', 'Mar 2020', 'East', 'A'), 'values_1'] > 0
 
 
-# test_pivot_wider_single_grouper {{{1
-def test_pivot_wider_single_grouper(t_sample_data):
+# test_pivot_table_single_grouper {{{1
+def test_pivot_table_single_grouper(t_sample_data):
     """
     """
     df = t_sample_data
 
-    p2 = pivot_wider(df, index=['dates', 'regions', 'ids'],
+    p2 = pivot_table(df, index=['dates', 'regions', 'ids'],
                      freq='Q', format_date=True)
 
     assert p2.loc[('Mar 2020', 'East', 'A'), 'values_1'] > 0
@@ -1203,7 +1206,7 @@ def test_rename_axis(t_sample_data):
     expected = ['AAA', 'BBB']
 
     df = t_sample_data
-    df = pivot_wider(df, index=['countries', 'regions'], values='values_1')
+    df = pivot_table(df, index=['countries', 'regions'], values='values_1')
     df = rename_axis(df, mapper=('AAA', 'BBB'), axis='rows')
     actual = df.index.names
 
@@ -1316,7 +1319,7 @@ def test_select_no_parms(t_sample_data):
 def test_select_str_with_regex(t_sample_data):
 
     df = t_sample_data
-    df = select(df, 'values', regex=True)
+    df = select(df, regex='values')
 
     expected = ['values_1', 'values_2']
 
@@ -1325,8 +1328,34 @@ def test_select_str_with_regex(t_sample_data):
     assert expected == actual
 
 
-# test_select_slice {{{1
-def test_select_slice(t_sample_data):
+# test_select_str_with_regex {{{1
+def test_select_str_with_like(t_sample_data):
+
+    df = t_sample_data
+    df = select(df, like='values')
+
+    expected = ['values_1', 'values_2']
+
+    actual = df.columns.tolist()
+
+    assert expected == actual
+
+
+# test_select_slice_with_integers {{{1
+def test_select_slice_with_integers(t_sample_data):
+
+    df = t_sample_data
+    df = select(df, (3 , 6))
+
+    expected = ['countries', 'regions', 'ids', 'values_1']
+
+    actual = df.columns.tolist()
+
+    assert expected == actual
+
+
+# test_select_slice_with_column_names {{{1
+def test_select_slice_with_column_names(t_sample_data):
 
     df = t_sample_data
     df = select(df, ('countries', 'values_1'))
@@ -1414,15 +1443,37 @@ def test_select_columns(t_sample_data):
     assert expected == actual
 
 
+# test_select_include {{{1
+def test_select_include(t_sample_data):
+
+    df = t_sample_data
+    df = select(df, include='number')
+
+    expected = ['values_1', 'values_2']
+    actual = df.columns.tolist()
+
+    assert expected == actual
+
+
+# test_select_exclude {{{1
+def test_select_exclude(t_sample_data):
+
+    df = t_sample_data
+    df = select(df, exclude='number')
+
+    expected = ['dates', 'order_dates', 'countries', 'regions', 'ids']
+    actual = df.columns.tolist()
+
+    assert expected == actual
+
+
 # test_select_invalid_column {{{1
 def test_select_invalid_column(t_sample_data):
 
     df = t_sample_data
 
-    expected = None
-    actual = select(df, 'AAA')
-
-    assert expected == actual
+    with pytest.raises(KeyError):
+        actual = select(df, 'AAA')
 
 
 # test_set_columns {{{1
