@@ -83,7 +83,7 @@ def across(df: pd.DataFrame,
     function
         function to be called.
     series_obj
-        Default is True.
+        Default is False.
         True - Function applied at Series or (DataFrame) 'object' level.
         False - Function applied to each Series row values.
 
@@ -219,10 +219,22 @@ def adorn(df: pd.DataFrame,
 
     Examples
     --------
+
     .. code-block::
 
-        import pandas as pd
-        from piper.pandas import *
+        df = sample_matrix(seed=42)
+        df = adorn(df, ['a', 'c'], axis='row')
+        head(df, 10, tablefmt='plain')
+
+               a       b    c       d        e
+        0     15   8.617   16  25.23     7.658
+        1      8  25.792   18   5.305   15.426
+        2      5   5.343   12  -9.133   -7.249
+        3      4  -0.128   13   0.92    -4.123
+        4     25   7.742   11  -4.247    4.556
+        All   57           70
+
+    .. code-block::
 
         url = 'https://github.com/datagy/pivot_table_pandas/raw/master/sample_pivot.xlsx'
         df = pd.read_excel(url, parse_dates=['Date'])
@@ -323,9 +335,6 @@ def adorn(df: pd.DataFrame,
 # assign() {{{1
 def assign(df: pd.DataFrame,
            *args,
-           lambda_str: bool = False,
-           lambda_var: str = 'x',
-           info: bool = False,
            **kwargs) -> pd.DataFrame:
     '''Assign new columns to a DataFrame.
 
@@ -334,16 +343,10 @@ def assign(df: pd.DataFrame,
 
     Parameters
     ----------
-    lambda_str
-        Default is False. Treat string as a the 'body' of a lambda function.
-        Assign function will convert the string to a lambda before passing to
-        pd.DataFrame.assign()
-    lambda_var
-        Default is 'x'.
-    info
-        Default is False.
-        if True - output contents of kwargs where keyword value is being
-        converted from string to lambda function. (For debug purposes)
+    df
+        pandas dataframe
+    args
+        arbitrary arguments (for future use)
     **kwargs
         dict of {str: callable or Series}
 
@@ -353,20 +356,34 @@ def assign(df: pd.DataFrame,
         values are not callable, (e.g. a Series, scalar, or array), they are
         simply assigned.
 
+        .. note::
+
+            If you wish to apply a function to a columns set of values:
+            pass a tuple with column name and function to call.
+            For example:
+
+        .. code-block::
+
+            assign(reversed=('regions', lambda x: x[::-1]))
+
+        is converted to:
+
+        .. code-block::
+
+            assign(reversed=lambda x: x['regions'].apply(lambda x: x[::-1]))
+
     Returns
     -------
     A pandas DataFrame
         A new DataFrame with the new columns in addition to
         all the existing columns.
 
-
     Notes
     -----
     Assigning multiple columns within the same ``assign`` is possible.
     Later items in '\\*\\*kwargs' may refer to newly created or modified
     columns in 'df'; items are computed and assigned into 'df' in order.
 
-
     Examples
     --------
     You can create a dictionary of column names to corresponding functions,
@@ -375,82 +392,63 @@ def assign(df: pd.DataFrame,
     .. code-block::
 
         %%piper --dot
+
         sample_data()
-        .. assign(**{'reversed': 'x.regions.apply(lambda x: x[::-1])',
-                     'v1_x_10': 'x.values_1 * 10',
+        .. assign(**{'reversed': ('regions', lambda x: x[::-1]),
+                     'v1_x_10': lambda x: x.values_1 * 10,
                      'v2_div_4': lambda x: x.values_2 / 4,
                      'dow': lambda x: x.dates.dt.day_name(),
                      'ref': lambda x: x.v2_div_4 * 5,
-                     'values_1': 'x.values_1.astype(int)',
-                     'values_2': 'x.values_2.astype(int)',
-                     'ids': lambda x: x.ids.astype('category')}, lambda_str=True)
-        .. pd.DataFrame.astype({'values_1': float, 'values_2': float})
+                     'ids': lambda x: x.ids.astype('category')})
+        .. across(['values_1', 'values_2'], lambda x: x.astype(float),
+                  series_obj=True)
         .. relocate('dow', 'after', 'dates')
         .. select(['-dates', '-order_dates'])
-        .. head()
+        .. head(tablefmt='plain')
 
-          dow        countries  regions  ids values_1  values_2   reversed v1_x_10  v2_div_4      ref
-          Wednesday  Italy      East     A        311        26   tsaE        3110      6.5     32.5
-          Thursday   Portugal   South    D        150       375   htuoS       1500     93.75   468.75
-          Friday     Spain      East     A        396        88   tsaE        3960     22      110
-          Saturday   Italy      East     B        319       233   tsaE        3190     58.25   291.25
+            dow       countries regions  ids values_1 values_2 reversed v1_x_10 v2_div_4  ref
+         0  Wednesday Italy     East     A        311       26 tsaE        3110        6   32
+         1  Thursday  Portugal  South    D        150      375 htuoS       1500       94  469
+         2  Friday    Spain     East     A        396       88 tsaE        3960       22  110
+         3  Saturday  Italy     East     B        319      233 tsaE        3190       58  291
 
     .. code-block::
 
         %%piper
+
         sample_sales()
         >> select()
         >> assign(month_plus_one = lambda x: x.month + pd.Timedelta(1, 'D'),
-                  alt_formula = "x.actual_sales * .2", lambda_str=True)
+                  alt_formula = lambda x: x.actual_sales * .2)
         >> select(['-target_profit'])
-        >> assign(profit_sales = 'x.actual_profit - x.actual_sales',
-               month_value = "x.month.dt.month",
-               product_added = "x['product'] + ', TEST'",
-               salesthing = "x.target_sales.sum()", lambda_str=True)
+        >> assign(profit_sales = lambda x: x.actual_profit - x.actual_sales,
+                  month_value = lambda x: x.month.dt.month,
+                  product_added = lambda x: x['product'] + 'TEST',
+                  salesthing = lambda x: x.target_sales.sum())
         >> select(['-month', '-actual_sales', '-actual_profit', '-month_value', '-location'])
         >> assign(profit_sales = lambda x: x.profit_sales.astype(int))
         >> reset_index(drop=True)
         >> head(tablefmt='plain')
 
-            product      target_sales  month_plus_one         alt_formula    profit_sales  product_added      salesthing
-         0  Beachwear           31749  2021-01-02 00:00:00           5842          -27456  Beachwear, TEST       5423715
-         1  Beachwear           37833  2021-01-02 00:00:00           6810          -28601  Beachwear, TEST       5423715
-         2  Jeans               29485  2021-01-02 00:00:00           6310          -27132  Jeans, TEST           5423715
-         3  Jeans               37524  2021-01-02 00:00:00           8180          -36811  Jeans, TEST           5423715
-
-    An example of assign accessing the apply function of a series (column), converting
-    the row values from a string of comma separated values into a list object.
-
-    .. code-block::
-
-        %%piper
-        sample_sales()
-        >> select(['-target_profit', '-month', '-location', '-target_sales'])
-        >> str_join(columns=['actual_sales', 'product', 'actual_profit'],
-                    sep=',', column='actual_sales',
-                    drop=False)
-        >> assign(actual_sales_= lambda x: x.actual_sales_.apply(lambda x: x.split(',')))
-        >> head(tablefmt='plain')
-
-             product   actual_sales  actual_sales_                        actual_profit
-          4  Beachwear        29209  ['29209.08', 'Beachwear', '1752.54']          1753
-        125  Beachwear        34050  ['34049.7', 'Beachwear', '5447.95']           5448
-         21  Jeans            31549  ['31548.95', 'Jeans', '4416.85']              4417
-        148  Jeans            40901  ['40901.16', 'Jeans', '4090.12']              4090
+            product   target_sales month_plus_one alt_formula profit_sales product_added salesthing
+         0  Beachwear        31749     2021-01-02        5842       -27456 BeachwearTEST    5423715
+         1  Beachwear        37833     2021-01-02        6810       -28601 BeachwearTEST    5423715
+         2  Jeans            29485     2021-01-02        6310       -27132 JeansTEST        5423715
+         3  Jeans            37524     2021-01-02        8180       -36811 JeansTEST        5423715
 
     '''
     if kwargs:
         for keyword, value in kwargs.items():
-            if lambda_str:
-                if isinstance(value, str):
-                    text = f"lambda {lambda_var}: " + value
-                    function = eval(text)
-                    if info:
-                        logger.info(f'keyword {keyword} = {lambda_str}')
+            if isinstance(value, tuple):
+                column_to_apply_function, function = value
+                new_function = lambda x: x[column_to_apply_function].apply(function)
+                kwargs[keyword] = new_function
 
-                    kwargs[keyword] = function
-
-    df = df.assign(*args, **kwargs)
+    try:
+        df = df.assign(*args, **kwargs)
+    except ValueError as e:
+        logger.info(e)
+        raise ValueError('''Try assign(column='column_to_apply_func', function)''')
 
     return df
 
@@ -587,7 +585,6 @@ def count(df: pd.DataFrame,
         E        3   60  100.0
         Total    5  100
 
-
     Parameters
     ----------
     df
@@ -619,14 +616,14 @@ def count(df: pd.DataFrame,
     # NOTE:: pd.groupby by default does not count nans!
     try:
         if isinstance(columns, str):
-            p1 = df.groupby(columns, dropna=False).agg(totals=(columns, lambda x: np.count_nonzero(x)))
+            p1 = df.groupby(columns, dropna=False).agg(totals=(columns, lambda x: x.value_counts(dropna=False)))
         elif isinstance(df, pd.Series):
             new_df = df.to_frame()
             columns = new_df.columns.tolist()[0]
-            p1 = new_df.groupby(columns, dropna=False).agg(totals=(columns, lambda x: np.count_nonzero(x)))
+            p1 = new_df.groupby(columns, dropna=False).agg(totals=(columns, lambda x: x.value_counts(dropna=False)))
         elif isinstance(df, pd.DataFrame):
             if columns is not None:
-                p1 = df.groupby(columns, dropna=False).agg(totals=(columns[0], lambda x: np.count_nonzero(x)))
+                p1 = df.groupby(columns, dropna=False).agg(totals=(columns[0], lambda x: x.value_counts(dropna=False)))
             else:
                 p1 = df.count().to_frame()
 
@@ -949,8 +946,9 @@ def drop(df: pd.DataFrame,
 
 # drop_columns() {{{1
 def drop_columns(df: pd.DataFrame,
-                 value: Union[str, int, float] = None) -> pd.DataFrame:
-    ''' drop columns containing blanks or zeros
+                 value: Union[str, int, float] = None,
+                 how: str = 'all') -> pd.DataFrame:
+    ''' drop columns containing blanks, zeros or na
 
     Examples
     --------
@@ -965,37 +963,33 @@ def drop_columns(df: pd.DataFrame,
     df
         pandas dataframe
     value
-        For each dataframe column, if the specified value is present within
-        EVERY row, drop the column.
+        Default is None. Drop a column if it contains a blank value in every row.
+        Enter a literal string or numeric value to check against.
 
-        Default is None which will drop a column if it contains blanks in every
-        row.
+        Special value - 'isna'. If specified, depending on the 'how' parameter
+        drop columns containing na / null values.
+    how
+        {None, 'any', 'all'}, default 'all'.
+        Determine if row or column is removed from DataFrame, when we have
+        at least one NA or all NA.
 
+        * 'any' : If any NA values are present, drop that row or column.
+        * 'all' : If all values are NA, drop that row or column.
 
     Returns
     -------
     A pandas DataFrame
     '''
-    if value is None:
-        value = "''"
-
-    if isinstance(value, int) or isinstance(value, float):
-        columns_to_check = df.select_dtypes(include='number').columns
+    if value == 'isna':
+        df = df.dropna(axis=1, how=how)
     else:
-        columns_to_check = df.select_dtypes(exclude='number').columns
+        if value is None:
+            value = ""
 
-    cols = []
-    for col in df.columns:
-
-        if col not in columns_to_check:
-            cols.append(col)
-        else:
-            # If not all records equal the value passed, keep column
-            keep = df.query(f"{col} != {value}").shape[0] == df.shape[0]
-            if keep:
-                cols.append(col)
-
-    df = df[cols]
+        for col in df.columns:
+            drop_column = np.where(df[col] == value, 1, 0).sum() == df.shape[0]
+            if drop_column:
+                df = df.drop(columns=col, axis=1)
 
     return df
 
@@ -1084,6 +1078,7 @@ def explode(df: pd.DataFrame,
 
     Examples
     --------
+
     .. code-block::
 
         from piper.factory import sample_data
@@ -1257,10 +1252,10 @@ def group_by(df: pd.DataFrame,
 
     Examples
     --------
+
     .. code-block::
 
         %%piper
-
         sample_data()
         >> where("ids == 'A'")
         >> where("values_1 > 300 & countries.isin(['Italy', 'Spain'])")
@@ -1397,10 +1392,11 @@ def info(df,
     nunique_list = [df[col].nunique() for col in df]
     dupes_list = [df.duplicated(col, keep=False).sum() for col in df]
     total_list = [df[col].value_counts(dropna=False).sum() for col in df]
+    infer_list = [pd.api.types.infer_dtype(df[col]) for col in df]
 
     dicta = {'columns': df.columns.values, 'type': dtypes_list,
-             'n': total_list, 'isna': na_list, 'isnull': null_list,
-             'unique': nunique_list}
+             'inferred': infer_list, 'n': total_list, 'isna': na_list,
+             'isnull': null_list, 'unique': nunique_list}
 
     if n_dupes:
         dicta.update({'n dupes': dupes_list})
@@ -1815,8 +1811,8 @@ def pivot_longer(df: pd.DataFrame,
     return df.melt(*args, **kwargs)
 
 
-# pivot_wider() {{{1
-def pivot_wider(df: pd.DataFrame,
+# pivot_table() {{{1
+def pivot_table(df: pd.DataFrame,
                 *args,
                 freq: str = 'M',
                 format_date: bool = False,
@@ -1890,6 +1886,21 @@ def relocate(df: pd.DataFrame,
 
     Examples
     --------
+    .. code-block::
+
+        %%piper
+        sample_matrix()
+        >> relocate('e', 'before', 'b')
+        >> select('-c')
+        >> where("a < 20 and b > 1")
+        >> order_by(['-d'])
+        >> head(5, tablefmt='plain')
+
+              a    e    b    d
+         0   15    8    9   25
+         1    8   15   26    5
+         2    5   -7    5   -9
+
     .. code-block::
 
         %%piper
@@ -2200,106 +2211,151 @@ def sample(df: pd.DataFrame,
 
 # select() {{{1
 def select(df: pd.DataFrame,
-           expr: str = None,
-           regex: bool = False) -> pd.DataFrame:
+           *args,
+           regex: str = None,
+           like: str = None,
+           include: str = None,
+           exclude: str = None) -> pd.DataFrame:
     '''select dataframe columns
 
-    Based on select() function from R tidyverse.
+    Inspired by the select() function from R tidyverse.
 
-    Given dataframe, select column names
+    Select column names from a dataframe
 
     Examples
     --------
+
     .. code-block::
 
+        df = sample_sales()
+
         select(df) # select ALL columns
 
         # select column column listed
-        select(df, 'column_name')
+        select(df, 'location')
 
         # select columns listed
-        select(df, ['column_name', 'other_column'])
+        select(df, ['product', 'target_sales'])
 
         # select ALL columns EXCEPT the column listed (identified by - minus sign prefix)
-        select(df, '-column_name')
+        select(df, '-target_profit')
 
         # select ALL columns EXCEPT the column specified with a minus sign within the list
-        select(df, ['-column_name', '-other_column'])
+        select(df, ['-target_sales', '-target_profit'])
 
-        # select column range from column up to and including the 'to' column.
-        # This is achieved by passing a 'tuple' -> e.g. ('from_col', 'to_col')
-        select(df, ('title', 'isbn'))
+        # select column range using a tuple from column up to and including the 'to' column.
+        select(df, ('month', 'actual_profit'))
+
+        # select all number data types
+        select(df, '-month', include='number')
+
+        # exclude all float data types including month column
+        select(df, '-month', exclude='float')
 
         # select using a regex string
-        select(df, 'value') -> select fields containing 'value'
-        select(df, '^value') -> select fields starting with 'value'
-        select(df, 'value$') -> select fields ending with 'value'
+        select(df, regex='sales') -> select fields containing 'sales'
+        select(df, regex='^sales') -> select fields starting with 'sales'
+        select(df, regex='sales$') -> select fields ending with 'sales'
+
+    .. note ::
+
+        See the numpy dtype hierarchy. To select:
+            - strings use the 'object' dtype, but note that this will return
+              all object dtype columns
+            - all numeric types, use np.number or 'number'
+            - datetimes, use np.datetime64, 'datetime' or 'datetime64'
+            - timedeltas, use np.timedelta64, 'timedelta' or 'timedelta64'
+            - Pandas categorical dtypes, use 'category'
+            - Pandas datetimetz dtypes, use 'datetimetz' (new in 0.20.0) or 'datetime64[ns, tz]'
 
 
     Parameters
     ----------
     df
         dataframe
-    expr
-        default None
-        str - single column (in quotes)
-        list -> list of column names (in quotes)
+    args
+        if not specified, then ALL columns are selected
+            - str   : single column (in quotes)
+            - list  : list of column names (in quotes)
+            - tuple : specify a range of column names or column positions (1-based)
+
+        .. note::
+
+            Prefixing column name with a minus sign filters out the column
+            from the returned list of columns e.g. '-sales', '-month'
 
-        NOTE: prefixing column name with a minus sign filters out the column
-        from returned list of columns
     regex
-        default False. If True, treat column string as a regex
+        Default None. Wrapper for regex keyword in pd.DataFrame.filter()
+        Keep labels from axis for which re.search(regex, label) == True.
+    like
+        Default None. Wrapper for like keyword in pd.DataFrame.filter()
+        Keep labels from axis for which like in label == True.
+    include
+        Default None. Wrapper for include keyword in pd.DataFrame.select_dtypes()
+    exclude
+        Default None. Wrapper for exclude keyword in pd.DataFrame.select_dtypes()
 
 
     Returns
     -------
     pandas DataFrame object
     '''
-    returned_column_list = df.columns.tolist()
+    columns = list(df.columns)
+    selected: List = []
+    drop: List = []
 
-    try:
+    for column_arg in args:
+        if isinstance(column_arg, str):
+            selected, drop = _check_col(column_arg, selected, drop, columns)
 
-        if expr is None:
-            return df
+        # Tuples used to specify a 'range' of columns (from/to)
+        if isinstance(column_arg, tuple):
 
-        if isinstance(expr, tuple):
-            return df.loc[:, slice(*expr)]
+            if sum([isinstance(v, str) for v in column_arg]) == 2:
+                cols = list(df.loc[:, slice(*column_arg)].columns)
+                for col in cols:
+                    selected, drop = _check_col(col, selected, drop, columns)
 
-        if isinstance(expr, str):
+            if sum([isinstance(v, int) for v in column_arg]) == 2:
+                first, last = column_arg
+                cols = list(df.iloc[:, range(first-1, last)].columns)
+                for col in cols:
+                    selected, drop = _check_col(col, selected, drop, columns)
 
-            if regex:
-                return df.filter(regex=expr)
+        # Lists to be used for passing in a set of distinct values
+        if isinstance(column_arg, list):
+            for col in column_arg:
+                selected, drop = _check_col(col, selected, drop, columns)
 
-            if expr.startswith('-'):
-                returned_column_list.remove(expr[1:])
-                return df[returned_column_list]
-            else:
-                if expr not in returned_column_list:
-                    raise KeyError(f"Column '{expr}' not found!")
+    if like is not None:
+        cols = df.filter(like=like)
+        for col in cols:
+             selected, drop = _check_col(col, selected, drop, columns)
 
-                return df[expr].to_frame()
+    if regex is not None:
+        cols = df.filter(regex=regex)
+        for col in cols:
+             selected, drop = _check_col(col, selected, drop, columns)
 
-        if isinstance(expr, list):
+    if include is not None:
+        cols = df.select_dtypes(include=include)
+        for col in cols:
+            selected, drop = _check_col(col, selected, drop, columns)
 
-            selected_cols = [x for x in expr if not x.startswith('-')]
-            if len(selected_cols) > 0:
-                return df[selected_cols]
+    if exclude is not None:
+        cols = df.select_dtypes(exclude=exclude)
+        for col in cols:
+            selected, drop = _check_col(col, selected, drop, columns)
 
-            remove_cols = [x[1:] for x in expr if x.startswith('-')]
-            if len(remove_cols) > 0:
-                logger.debug(remove_cols)
-                logger.debug(len(remove_cols))
+    if selected == []:
+        selected = columns
 
-                returned_column_list = [x for x in returned_column_list if x not in remove_cols]
-
-                for col in returned_column_list:
-                    returned_column_list
-
-                return df[returned_column_list]
-
-    except (ValueError, KeyError) as e:
-        logger.info(e)
+    if drop != []:
+        for col in drop:
+            if col in selected:
+                selected.remove(col)
 
+    return df[selected]
 
 # set_columns() {{{1
 def set_columns(df: pd.DataFrame,
@@ -2540,7 +2596,7 @@ def str_split(df: pd.DataFrame,
               drop: bool = False) -> pd.DataFrame:
     ''' split column
 
-    Function accepts a column to split, a pattern/delimitter values and optional
+    Function accepts a column to split, a pattern/delimitter value and optional
     list of column names to store the result. By default the result is placed just
     after the specified column.
 
@@ -2564,6 +2620,11 @@ def str_split(df: pd.DataFrame,
         column values
     pat
         regular expression pattern. Default is ','
+
+        .. note::
+
+            For space(s), safer to use r'\\s' rather than ' '.
+
     n
         default -1. Number of splits to capture. -1 means capture ALL splits
     loc
@@ -2724,6 +2785,20 @@ def summarise(df: pd.DataFrame,
     The equivalent to:
     %pipe df >> group_by(['col1', 'col2']) >> summarise(sum)
 
+    .. code-block::
+
+        %%piper
+        sample_sales()
+        >> group_by(['location', 'product'])
+        >> summarise()
+        >> head(tablefmt='plain')
+
+                                    target_sales    target_profit    actual_sales    actual_profit
+        ('London', 'Beachwear')           407640            41372          388762            38940
+        ('London', 'Footwear')            275605            21449          274674            21411
+        ('London', 'Jeans')               414488            42200          404440            40691
+        ('London', 'Sportswear')          293384            28149          291561            28434
+
 
     Examples
     --------
@@ -3178,6 +3253,50 @@ def where(df: pd.DataFrame,
     except (pd.core.computation.ops.UndefinedVariableError) as e:
         logger.info("ERROR: External @variable? use: where(..., global_dict=globals())")
 
+# _check_col() {{{1
+def _check_col(column: str,
+               selected: List,
+               drop: List,
+               columns: List) -> Tuple[List, List]:
+    ''' Check column add/remove from column list
+
+    Used by select() function.
+
+    Parameter
+    ---------
+    column
+        column to be added/removed
+    selected
+        list of columns to be selected
+    drop
+        list of columns to be dropped
+    columns
+        all columns within dataframe (for validation)
+
+    Returns
+    -------
+    list
+        updated select column list
+    '''
+    remove = False
+
+    if isinstance(column, str):
+        if column.startswith('-'):
+            remove = True
+            column = column[1:]
+
+    if column not in columns:
+        raise KeyError(f"Column '{column}' not found!")
+
+    if remove:
+        if column not in drop:
+            drop.append(column)
+    else:
+        if column not in selected:
+            selected.append(column)
+
+    return selected, drop
+
 
 # _set_grouper() {{{1
 def _set_grouper(df: pd.DataFrame,
